﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using VillainmadConfig;

namespace VMGame
{
    class VMStage : Stage
    {
        #region Pause Menu
        public class PauseMenu : Menu
        {
            private static Color bgColor = new Color(0f, 0f, 0f, 0.5f);

            private class MenuContinue : MenuItem
            {
                public override void Select()
                {
                    Menu.Close();
                }

                public override void Draw(int frame, bool selected)
                {
                    Stage.instance.DrawString(GameState.instance.Game.fntGeneric,
                        "CONTINUE", new Vector2(320, 160),
                        (selected && Menu.TakeInput) ? Color.White : Color.Gray);
                }

                public MenuContinue(Menu Menu)
                    : base(Menu)
                {
                }
            }

            private class MenuExit : MenuItem
            {
                public override void Select()
                {
                    Menu.Game.Exit();
                }

                public override void Draw(int frame, bool selected)
                {
                    Stage.instance.DrawString(GameState.instance.Game.fntGeneric,
                        "QUIT", new Vector2(320, 200),
                        (selected && Menu.TakeInput) ? Color.White : Color.Gray);
                }

                public MenuExit(Menu Menu)
                    : base(Menu)
                {
                }
            }

            private List<MenuItem> _menuItems = new List<MenuItem>();

            public override List<MenuItem> Items
            {
                get
                {
                    return _menuItems;
                }
            }

            protected override void OnClose()
            {
                Stage.instance.Control.PlayBGM();
            }

            public PauseMenu()
            {
                Stage.instance.Control.PauseBGM();
                _menuItems.Add(new MenuContinue(this));
                _menuItems.Add(new MenuExit(this));
                TakeInput = false;
                QueueCommand(5, delegate() { TakeInput = true;});
            }

            public override void Draw(GameTime gameTime)
            {
                Stage.instance.Control.DrawRectangle(Stage.instance.GameArea,
                        bgColor);
                base.Draw(gameTime);
            }
        }
        #endregion

        #region Types
        /// <summary>
        /// Type of BGImg.
        /// </summary>
        public class BGImgType : GameEntity
        {
            public override string Image
            {
                get { return "background/Fog.png"; }
            }

            public override void Draw()
            {
                Vector2 tor = Vector2.Zero;
                for (int y = 0; y <= 3; y++)
                {
                    //I have no idea why this works, it just does.
                    tor.Y = (y * 64 + (Stage.Frame % 64)) * 4 - 256;
                    tor.X = 0;
                    Stage.DrawObject(Image, tor, Color.White);
                    tor.X = 256;
                    Stage.DrawObject(Image, tor, Color.White);
                }
            }
        }
        /// <summary>
        /// Instance of BGImg.
        /// </summary>
        /*
        public class RedBall : Ball
        {
            static Color Red = new Color(255, 128, 128);

            public override Color Color { get { return Red; } }

            public override void Start()
            {
            }
        }
        //*/
        public class Lexy : PlayerCharacter
        {
            private byte Xpos = 2;
            private bool Focused = false;
            public override float Speed
            {
                get
                {
                    return (Focused) ? 1.25F : 4;
                }
            }

            private string[] curImg = new string[]
            {"sprites/LexyLeft2.png", "sprites/LexyLeft1.png", 
                "sprites/LexyNeutral.png",
                "sprites/LexyRight1.png", "sprites/LexyRight2.png"};

            public override void Draw()
            {
                Stage.DrawObject(curImg[Xpos], Pos.Round(), Color.White);
                if (Focused)
                    Stage.DrawObject("sprites/hitbox.png", Pos.Round(), Color.White);
            }

            public override void Input(List<GameInput> input, 
                List<GameInput> pressed, List<GameInput> released)
            {
                bool gLeft = input.Contains(GameInput.DLeft);
                bool gRight = input.Contains(GameInput.DRight);
                if (gRight && Xpos < 4) Xpos++;
                else if (gLeft && Xpos > 0) Xpos--;
                else if (!gLeft && !gRight)
                {
                    if (Xpos > 2) Xpos--;
                    if (Xpos < 2) Xpos++;
                }
                Focused = (input.Contains(GameInput.Focus));

                base.Input(input, pressed, released);
            }
        }

        public class Ball : GameEntity
        {
            static Color Red = new Color(255, 128, 128);

            public override string Image { get { return "BulWobble"; } }
            //public override string Image { get { return "sprites/bulOrb.png"; } }

            //public override BlendState Blend { get {return GameController.screen;}}

            public override Color Color
            {
                get
                {
                    return _color;
                }
            }

            private Color _color = Color.White;

            public override float Size { get { return 32F; } }

            public override EntType Type { get { return EntType.ANIMATION; } }

            public override bool AnimStartZero { get { return true; } }

            public override void Start()
            {
                SwitchWhite();
            }

            private void SwitchRed()
            {
                _color = Red;
                Stage.QueueCommand(300, delegate() { SwitchWhite(); });
            }

            private void SwitchWhite()
            {
                _color = Color.White;
                Stage.QueueCommand(300, delegate() { SwitchRed(); });
            }
        }

        public class BlahType : GameEntity
        {
            static Color Blue = new Color(0.25F, 0.25F, 1F);
            static Color Red = new Color(1F, 0.25F, 0.25F);

            public override string Image
            {
                get { return "sprites/whiteSquare.png"; }
            }

            public override BlendState Blend { get { return BlendState.Additive; } }

            public override bool RotMove { get { return true; } }

            public override float Size { get { return 32F; } }

            public override void Draw()
            {
                Stage.DrawObject(Image, Pos, null, Red, Rot, 1F,
                    SpriteEffects.None);
                Stage.DrawObject("sprites/bulOrb.png", Pos, null, Color.White,
                    Rot, 0.25F, SpriteEffects.None);
            }
        }
        #endregion

        #region Values
        public override int PlayerLayer { get { return 9; } }
        
        public override Dictionary<string, Animation> animationList()
        {
            Dictionary<string, Animation> animationList = new Dictionary<string, 
                Animation>();

            animationList.Add("BulWobble", new Animation(new AnimFrame[]{
                new AnimFrame(Control, "sprites/bulOrbSheet.png", 1F, 
                    new Rectangle(0, 0, 16, 16), SpriteEffects.None),
                new AnimFrame(Control, "sprites/bulOrbSheet.png", 1F, 
                    new Rectangle(16, 0, 16, 16), SpriteEffects.None),
                new AnimFrame(Control, "sprites/bulOrbSheet.png", 1F, 
                    new Rectangle(32, 0, 16, 16), SpriteEffects.None),
                new AnimFrame(Control, "sprites/bulOrbSheet.png", 1F, 
                    new Rectangle(48, 0, 16, 16), SpriteEffects.None)
            }, 4));

            return animationList;
        }

        public override Rectangle PlayerField
        {
            get
            {
                return new Rectangle(20, 20, 
                    GameArea.Width - 40, GameArea.Height - 40);
            }
        }

        public override int ObjTypes
        {
            get { return 16; }
        }

        public override string[] textureList
        { get
            {
                return new string[]
                {
                    "sprites/bulOrb.png", 
                    "sprites/bulOrbSheet.png",
                    "background/Fog.png", 
                    "sprites/whiteSquare.png"
               };
        } }

        public override string[] soundList
        {
            get
            {
                return new string[] {
                    "Tiptap.wav",
                    "BGM\\loopytesty.wav"
                };
            }
        }

        //public override Point UpperLeft { get { return new Point(128, 16); } }

        //public override Point LowerRight { get { return new Point(384, 488); } }

        public override Rectangle GameArea
        {
            get
            {
                return new Rectangle(128, 16, 384, 448);
            }
        }

        public override Vector2 PlayerStart
        { get { return new Vector2(GameArea.Width / 2, GameArea.Height - 128); } }
        #endregion

        #region Constructor
        public VMStage()
        {
            StartCenter = new Vector2(GameArea.Width / 2, GameArea.X);
        }
        #endregion

        public override void Start()
        {
            Control.PlayBGM("BGM\\loopytesty.wav");

            AddPlayer<Lexy>();

            AddObject<BGImgType>(Vector2.Zero, Vector2.Zero, 0F, 0);
            EntityRef Blah = AddObject<BlahType>(new Vector2(0F, 128F), 
                new Vector2(0.3F, 0.01F), 0F, 12);
            QueueCommand(120, delegate() { Shenanigans(Blah); }, Blah);

            //QueueCommand(1800, delegate() { Control.StartStage<VMStage>(); });
            Rotate();
        }

        float Rot = 0;

        Vector2 StartCenter;

        public void Rotate()
        {
            AddObject<Ball>(StartCenter, X10s.FromAngleDeg(Rot, 0.3F), 0F, 11);
            AddObject<Ball>(StartCenter, X10s.FromAngleDeg(Rot + 180, 0.3F), 0F, 11);
            Rot += 5.5F;
            if (Rot > 360) Rot -= 360;
            QueueCommand(3, delegate()
            {
                Rotate();
            });
        }

        public void Shenanigans(EntityRef curID)
        {
            MoveTo(curID, new Vector2(128, 192), 60);
            QueueCommand(90, delegate()
            {
                curID.Entity.Vel = new Vector2(2, -2);
            });
        }

        public override void Input(List<GameInput> input, List<GameInput> pressed,
            List<GameInput> released)
        {
            if (pressed.Contains(GameInput.Esc))
            {
                Control.AddMenu<PauseMenu>();
            }

            base.Input(input, pressed, released);
        } 
    }
}
